@@ -36,6 +36,7 @@ except ImportError:
     ResponseOutputMessageParam = Dict[str, Any]
     RESPONSES_AVAILABLE = False
 
+from lmnr.opentelemetry_lib.decorators import json_dumps
 from lmnr.opentelemetry_lib.tracing.context import (
     get_current_context,
     get_event_attributes_from_context,
@@ -139,6 +140,10 @@ class TracedData(pydantic.BaseModel):
     request_model: Optional[str] = pydantic.Field(default=None)
     response_model: Optional[str] = pydantic.Field(default=None)
 
+    # Reasoning attributes
+    request_reasoning_summary: Optional[str] = pydantic.Field(default=None)
+    request_reasoning_effort: Optional[str] = pydantic.Field(default=None)
+
 
 responses: dict[str, TracedData] = {}
 
@@ -204,7 +209,28 @@ def set_data_attributes(traced_response: TracedData, span: Span):
                 SpanAttributes.LLM_USAGE_CACHE_READ_INPUT_TOKENS,
                 usage.input_tokens_details.cached_tokens,
             )
-        # TODO: add reasoning tokens in output token details
+
+        reasoning_tokens = None
+        if usage.output_tokens_details:
+            reasoning_tokens = usage.output_tokens_details.reasoning_tokens
+
+        _set_span_attribute(
+            span,
+            SpanAttributes.LLM_USAGE_REASONING_TOKENS,
+            reasoning_tokens or 0,
+        )
+
+    _set_span_attribute(
+        span,
+        f"{SpanAttributes.LLM_REQUEST_REASONING_SUMMARY}",
+        traced_response.request_reasoning_summary or (),
+    )
+
+    _set_span_attribute(
+        span,
+        f"{SpanAttributes.LLM_REQUEST_REASONING_EFFORT}",
+        traced_response.request_reasoning_effort or (),
+    )
 
     if should_send_prompts():
         prompt_index = 0
@@ -275,7 +301,9 @@ def set_data_attributes(traced_response: TracedData, span: Span):
                     prompt_index += 1
                 elif block_dict.get("type") == "computer_call_output":
                     _set_span_attribute(
-                        span, f"{GEN_AI_PROMPT}.{prompt_index}.role", "computer-call"
+                        span,
+                        f"{GEN_AI_PROMPT}.{prompt_index}.role",
+                        "computer_call_output",
                     )
                     output_image_url = block_dict.get("output", {}).get("image_url")
                     if output_image_url:
@@ -299,16 +327,45 @@ def set_data_attributes(traced_response: TracedData, span: Span):
                     call_content = {}
                     if block_dict.get("id"):
                         call_content["id"] = block_dict.get("id")
-                    if block_dict.get("call_id"):
-                        call_content["call_id"] = block_dict.get("call_id")
                     if block_dict.get("action"):
                         call_content["action"] = block_dict.get("action")
                     _set_span_attribute(
                         span,
-                        f"{GEN_AI_PROMPT}.{prompt_index}.content",
+                        f"{GEN_AI_PROMPT}.{prompt_index}.tool_calls.0.arguments",
                         json.dumps(call_content),
                     )
+                    _set_span_attribute(
+                        span,
+                        f"{GEN_AI_PROMPT}.{prompt_index}.tool_calls.0.id",
+                        block_dict.get("call_id"),
+                    )
+                    _set_span_attribute(
+                        span,
+                        f"{GEN_AI_PROMPT}.{prompt_index}.tool_calls.0.name",
+                        "computer_call",
+                    )
                     prompt_index += 1
+                elif block_dict.get("type") == "reasoning":
+                    reasoning_summary = block_dict.get("summary")
+                    if reasoning_summary and isinstance(reasoning_summary, list):
+                        processed_chunks = [
+                            {"type": "text", "text": chunk.get("text")}
+                            for chunk in reasoning_summary
+                            if isinstance(chunk, dict)
+                            and chunk.get("type") == "summary_text"
+                        ]
+                        _set_span_attribute(
+                            span,
+                            f"{GEN_AI_PROMPT}.{prompt_index}.reasoning",
+                            json_dumps(processed_chunks),
+                        )
+                        _set_span_attribute(
+                            span,
+                            f"{GEN_AI_PROMPT}.{prompt_index}.role",
+                            "assistant",
+                        )
+                    # reasoning is followed by other content parts in the same messge,
+                    # so we don't increment the prompt index
                 # TODO: handle other block types
 
         _set_span_attribute(span, f"{GEN_AI_COMPLETION}.0.role", "assistant")
@@ -381,9 +438,19 @@ def set_data_attributes(traced_response: TracedData, span: Span):
                 )
                 tool_call_index += 1
             elif block_dict.get("type") == "reasoning":
-                _set_span_attribute(
-                    span, f"{GEN_AI_COMPLETION}.0.reasoning", block_dict.get("summary")
-                )
+                reasoning_summary = block_dict.get("summary")
+                if reasoning_summary and isinstance(reasoning_summary, list):
+                    processed_chunks = [
+                        {"type": "text", "text": chunk.get("text")}
+                        for chunk in reasoning_summary
+                        if isinstance(chunk, dict)
+                        and chunk.get("type") == "summary_text"
+                    ]
+                    _set_span_attribute(
+                        span,
+                        "gen_ai.completion.0.reasoning",
+                        json_dumps(processed_chunks),
+                    )
             # TODO: handle other block types, in particular other calls
 
 
@@ -423,6 +490,12 @@ def responses_get_or_create_wrapper(tracer: Tracer, wrapped, instance, args, kwa
                     "model", existing_data.get("request_model", "")
                 ),
                 response_model=existing_data.get("response_model", ""),
+                request_reasoning_summary=kwargs.get("reasoning", {}).get(
+                    "summary", existing_data.get("request_reasoning_summary")
+                ),
+                request_reasoning_effort=kwargs.get("reasoning", {}).get(
+                    "effort", existing_data.get("request_reasoning_effort")
+                ),
             )
         except Exception:
             traced_data = None
@@ -469,9 +542,16 @@ def responses_get_or_create_wrapper(tracer: Tracer, wrapped, instance, args, kwa
             ),
             request_model=existing_data.get("request_model", kwargs.get("model")),
             response_model=existing_data.get("response_model", parsed_response.model),
+            request_reasoning_summary=existing_data.get(
+                "request_reasoning_summary", kwargs.get("reasoning", {}).get("summary")
+            ),
+            request_reasoning_effort=existing_data.get(
+                "request_reasoning_effort", kwargs.get("reasoning", {}).get("effort")
+            ),
         )
         responses[parsed_response.id] = traced_data
     except Exception:
+        raise
         return response
 
     if parsed_response.status == "completed":
@@ -521,6 +601,12 @@ async def async_responses_get_or_create_wrapper(
                 output_text=kwargs.get("output_text", existing_data.get("output_text")),
                 request_model=kwargs.get("model", existing_data.get("request_model")),
                 response_model=existing_data.get("response_model"),
+                request_reasoning_summary=kwargs.get("reasoning", {}).get(
+                    "summary", existing_data.get("request_reasoning_summary")
+                ),
+                request_reasoning_effort=kwargs.get("reasoning", {}).get(
+                    "effort", existing_data.get("request_reasoning_effort")
+                ),
             )
         except Exception:
             traced_data = None
@@ -567,6 +653,12 @@ async def async_responses_get_or_create_wrapper(
             ),
             request_model=existing_data.get("request_model", kwargs.get("model")),
             response_model=existing_data.get("response_model", parsed_response.model),
+            request_reasoning_summary=existing_data.get(
+                "request_reasoning_summary", kwargs.get("reasoning", {}).get("summary")
+            ),
+            request_reasoning_effort=existing_data.get(
+                "request_reasoning_effort", kwargs.get("reasoning", {}).get("effort")
+            ),
         )
         responses[parsed_response.id] = traced_data
     except Exception:
